@@ -26,8 +26,8 @@
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-#include "MediaInfo/Reader/Reader_libcurl.h"
 #include "MediaInfo/File__Analyze.h"
+#include "MediaInfo/Reader/Reader_libcurl.h"
 #if defined MEDIAINFO_LIBCURL_DLL_RUNTIME
     //Copy of cURL include files
     #include "MediaInfo/Reader/Reader_libcurl_Include.h"
@@ -76,15 +76,13 @@ namespace Http
             Input           = Input.substr(Begin, Input.size() - Begin);
         }
     }
-    static void CutTail(std::string &Input, std::string &Output, const std::string &Delimiter, bool KeepDelimiter=false)
+    static void CutTail(std::string &Input, std::string &Output, const std::string &Delimiter)
     {
         // Remove the delimiter and everything that follows
         size_t Delimiter_Pos = Input.find(Delimiter);
         if (Delimiter_Pos != std::string::npos)
         {
-            size_t Begin    = Delimiter_Pos;
-            if (!KeepDelimiter)
-                Begin+=Delimiter.size();
+            size_t Begin    = Delimiter_Pos + Delimiter.size();
             size_t End      = Input.size() - Begin;
             Output          = Input.substr(Begin, End);
             Input           = Input.substr(0, Delimiter_Pos);
@@ -98,11 +96,36 @@ namespace Http
             : Host(In)
         {
             CutHead (Host,  Protocol,   "://"   );
-            CutTail (Host,  Search,     "?"     );
+            CutTail (Host,  Query,      "?"     );
+            CutTail (Query, Fragment,   "#"     );
             CutHead (Host,  User,       "@"     );
-            CutTail (Host,  Path,       "/",    true);
+            CutTail (Host,  Path,       "/"     );
             CutTail (User,  Password,   ":"     );
             CutTail (Host,  Port,       ":"     );
+            if (User.find('/')!=(size_t)-1 && Password.empty() && Path.empty())
+            {
+                // Something was weird, trying another method
+                Host=In;
+                CutHead (Host,  Protocol,   "://"   );
+                CutTail (Host,  Query,      "?"     );
+                CutTail (Query, Fragment,   "#"     );
+                CutTail (Host,  Path,       "/"     );
+                CutHead (Host,  User,       "@"     );
+                CutTail (User,  Password,   ":"     );
+                CutTail (Host,  Port,       ":"     );
+                if (Port.find_first_not_of("0123456789")!=(size_t)-1)
+                {
+                    // Format not understood, putting all in Protocol
+                    Protocol=In;
+                    User.clear();
+                    Password.clear();
+                    Host.clear();
+                    Port.clear();
+                    Path.clear();
+                    Query.clear();
+                    Fragment.clear();
+                }
+            }
 
             std::transform(Protocol.begin(), Protocol.end(), Protocol.begin(), ::tolower);
         }
@@ -118,8 +141,11 @@ namespace Http
             if (!User.empty() || !Password.empty())
             {
                 ToReturn += User;
-                ToReturn += ':';
-                ToReturn += Password;
+                if (!Password.empty())
+                {
+                    ToReturn += ':';
+                    ToReturn += Password;
+                }
                 ToReturn += '@';
             }
             ToReturn += Host;
@@ -128,13 +154,19 @@ namespace Http
                 ToReturn += ':';
                 ToReturn += Port;
             }
-            if (!Path.empty() || !Search.empty())
+            if (!Path.empty() || !Query.empty() || !Fragment.empty())
             {
+                ToReturn += '/';
                 ToReturn += Path;
-                if (!Search.empty())
+                if (!Query.empty())
                 {
                     ToReturn += '?';
-                    ToReturn += Search;
+                    ToReturn += Query;
+                }
+                if (!Fragment.empty())
+                {
+                    ToReturn += '#';
+                    ToReturn += Fragment;
                 }
             }
             return ToReturn;
@@ -147,17 +179,162 @@ namespace Http
         std::string Host;
         std::string Port;
         std::string Path;
-        std::string Search;
+        std::string Query;
+        std::string Fragment;
     };
 }
 
-Ztring Reader_libcurl_FileNameWithoutPassword(const Ztring &FileName)
+//---------------------------------------------------------------------------
+inline void PercentEncode_Add(string& In, char Char)
+{
+    unsigned char v=(unsigned char)Char;
+    unsigned char h=v>>4;
+    unsigned char l=v&0xF;
+    In+='%';
+    In+=(char)((h>=10?'A'-10:'0')+h);
+    In+=(char)((l>=10?'A'-10:'0')+l);
+}
+string PercentEncode(const string& In)
+{
+    string Result;
+    for (string::size_type i=0; i<In.size(); i++)
+    {
+        char C=In[i];
+        if ((C>='0' && C<='9')
+         || (C>='A' && C<='Z')
+         || (C>='a' && C<='z')
+         || C=='/' // Directory separator
+         || C=='-'
+         || C=='.'
+         || C=='_'
+         || C=='~')
+            Result+=In[i];
+        else
+            PercentEncode_Add(Result, C);
+    }
+    return Result;
+}
+string PercentDecode(const string& In)
+{
+    string Result;
+    Result.reserve(In.size());
+    for (string::size_type i=0; i<In.size(); i++)
+    {
+        char C=In[i];
+        switch (C)
+        {
+            case '%':
+            {
+                if (i+2>In.size())
+                    return In;
+                char C1=In[++i];
+                if (!((C1>='0' && C1<='9')
+                   || (C1>='A' && C1<='F')
+                   || (C1>='a' && C1<='f')))
+                    return In;
+                char C2=In[++i];
+                if (!((C2>='0' && C2<='9')
+                   || (C2>='A' && C2<='F')
+                   || (C2>='a' && C2<='f')))
+                    return In;
+                if (C1>='a')
+                    C1-='a'-10;
+                else if (C1>='A')
+                    C1-='A'-10;
+                else
+                    C1-='0';
+                if (C2>='a')
+                    C2-='a'-10;
+                else if (C2>='A')
+                    C2-='A'-10;
+                else
+                    C2-='0';
+                Result+=(const char)((C1<<4)|C2);
+                continue;
+            }
+            default:
+                Result+=C;
+        }
+    }
+    return Result;
+}
+MediaInfo_Config::urlencode DetectPercentEncode(const string& In, bool AcceptSlash=false)
+{
+    MediaInfo_Config::urlencode Result=MediaInfo_Config::URLEncode_Guess;
+    for (string::size_type i=0; i<In.size(); i++)
+    {
+        char C=In[i];
+        switch (C)
+        {
+            case '%':
+            {
+                if (i+2>In.size())
+                    return MediaInfo_Config::URLEncode_No;
+                char C1=In[++i];
+                if (!((C1>='0' && C1<='9')
+                   || (C1>='A' && C1<='F')
+                   || (C1>='a' && C1<='f')))
+                    return MediaInfo_Config::URLEncode_No;
+                char C2=In[++i];
+                if (!((C2>='0' && C2<='9')
+                   || (C2>='A' && C2<='F')
+                   || (C2>='a' && C2<='f')))
+                    return MediaInfo_Config::URLEncode_No;
+                Result=MediaInfo_Config::URLEncode_Yes;
+                continue;
+            }
+            case '+': // Often used as space separator
+            case '\'':
+            case '!':
+            case '(':
+            case ')':
+            case '*':
+                continue;
+            case ';':
+            case ':':
+            case '@':
+            case '&':
+            case '=':
+            case '$':
+            case ',':
+            case '?':
+            case '#':
+            case '[':
+            case ']':
+                return MediaInfo_Config::URLEncode_No;
+            case '/':
+                if (AcceptSlash)
+                    continue;
+                return MediaInfo_Config::URLEncode_No;
+            default:
+                if (!((C>='0' && C<='9')
+                   || (C>='A' && C<='Z')
+                   || (C>='a' && C<='z')
+                   || C=='-'
+                   || C=='.'
+                   || C=='_'
+                   || C=='~'))
+                {
+                    if (Result!=MediaInfo_Config::URLEncode_Yes)
+                        Result=MediaInfo_Config::URLEncode_No;
+                }
+        }
+    }
+    return Result;
+}
+
+Ztring Reader_libcurl_FileNameWithoutPasswordAndParameters(const Ztring &FileName)
 {
     Ztring FileName_Modified(FileName);
     size_t Begin=FileName_Modified.find(__T(':'), 6);
     size_t End=FileName_Modified.find(__T('@'));
     if (Begin!=string::npos && End!=string::npos && Begin<End)
         FileName_Modified.erase(Begin, End-Begin);
+
+    size_t Parameters_Begin=FileName_Modified.find(__T('?'));
+    if (Parameters_Begin!=string::npos)
+        FileName_Modified.erase(Parameters_Begin);
+
     return FileName_Modified;
 }
 
@@ -247,7 +424,7 @@ size_t libcurl_WriteData_CallBack_Amazon_AWS_Region(void *ptr, size_t size, size
     long http_code=0;
     if (curl_easy_getinfo(((Reader_libcurl_curl_data_getregion*)data)->Curl, CURLINFO_RESPONSE_CODE, &http_code)!=CURLE_OK || http_code!=200)
     {
-        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(((Reader_libcurl_curl_data_getregion*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
+        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPasswordAndParameters(((Reader_libcurl_curl_data_getregion*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
         return size*nmemb;
     }
 
@@ -269,7 +446,7 @@ size_t libcurl_WriteData_CallBack_Amazon_AWS_Region(void *ptr, size_t size, size
     //Error is displayed if region is not filled
     if (((Reader_libcurl_curl_data_getregion*)data)->Amazon_AWS_Region.empty())
     {
-        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(((Reader_libcurl_curl_data_getregion*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
+        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPasswordAndParameters(((Reader_libcurl_curl_data_getregion*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
         return 0;
     }
 
@@ -277,7 +454,7 @@ size_t libcurl_WriteData_CallBack_Amazon_AWS_Region(void *ptr, size_t size, size
 }
 
 //---------------------------------------------------------------------------
-void Amazon_AWS_Sign(MediaInfoLib::String &File_Name, struct curl_slist* &HttpHeader, const Http::Url &File_URL, const string &regionName, const string &serviceName, const ZtringList& ExternalHttpHeaders)
+void Amazon_AWS_Sign(struct curl_slist* &HttpHeader, const Http::Url &File_URL, const string &regionName, const string &serviceName, const ZtringList& ExternalHttpHeaders)
 {
     // Amazon AWS Authorization Header v4
     // See http://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-header-based-auth.html
@@ -299,12 +476,24 @@ void Amazon_AWS_Sign(MediaInfoLib::String &File_Name, struct curl_slist* &HttpHe
 
     //CanonicalRequest
     string CanonicalRequest;
-    CanonicalRequest+="GET\n";
-    CanonicalRequest+=File_URL.Path+'\n';
-    if (!File_URL.Search.empty())
+    CanonicalRequest+="GET\n/";
+    for (size_t i=0;;)
     {
-        CanonicalRequest+=File_URL.Search;
-        if (File_URL.Search[File_URL.Search.size()-1]!='=')
+        size_t j=File_URL.Path.find_first_of("!$&\'()*+,;=:@", i); //We need to percent-encode these chars as AWS provides them in their UI but requests percent-encode chars in the signature
+        CanonicalRequest+=File_URL.Path.substr(i, j-i);
+        if (j==(size_t)-1)
+            break;
+        char C=File_URL.Path[j];
+        if (C=='+')
+            C=' '; // "+" must be considered as " " in AWS
+        PercentEncode_Add(CanonicalRequest, C);
+        i=j+1;
+    }
+    CanonicalRequest+='\n';
+    if (!File_URL.Query.empty())
+    {
+        CanonicalRequest+=File_URL.Query;
+        if (File_URL.Query[File_URL.Query.size()-1]!='=')
             CanonicalRequest+='=';
     }
     CanonicalRequest+='\n';
@@ -361,26 +550,36 @@ void Amazon_AWS_Sign(MediaInfoLib::String &File_Name, struct curl_slist* &HttpHe
 };
 
 //---------------------------------------------------------------------------
-string Amazon_AWS_GetRegion(const string &serviceName, const string &bucketName, const Http::Url &File_URL, CURL* Curl, const ZtringList& ExternalHttpHeaders)
+size_t Amazon_AWS_GetRegion(const string &serviceName, const string &bucketName, const Http::Url &File_URL, Reader_libcurl* Reader_Curl, const ZtringList& ExternalHttpHeaders, string& RegionName)
 {
     //Transform the URL to a location request URL
     Http::Url File_URL2=File_URL;
     File_URL2.Host.erase(0, bucketName.size()+1);
     File_URL2.Path='/'+bucketName;
-    File_URL2.Search="location";
+    File_URL2.Query="location";
 
     //Send a location request
-    Reader_libcurl_curl_data_getregion Curl_Data2;
-    Curl_Data2.Curl=Curl;
-    Curl_Data2.File_Name.From_UTF8(File_URL2.ToString());
     struct curl_slist* HttpHeader=NULL;
-    Amazon_AWS_Sign(Curl_Data2.File_Name, HttpHeader, File_URL2, "us-east-1", serviceName, ExternalHttpHeaders);
+    Amazon_AWS_Sign(HttpHeader, File_URL2, "us-east-1", serviceName, ExternalHttpHeaders);
+    Reader_libcurl_curl_data_getregion Curl_Data2;
+    CURL* Curl=Reader_Curl->Curl_Data->Curl;
+    Curl_Data2.Curl=Curl;
+    File_URL2.User.clear();
+    File_URL2.Password.clear();
+    Curl_Data2.File_Name.From_UTF8(File_URL2.ToString());
     string FileName_String=Curl_Data2.File_Name.To_UTF8();
     curl_easy_setopt(Curl, CURLOPT_WRITEFUNCTION, &libcurl_WriteData_CallBack_Amazon_AWS_Region);
     curl_easy_setopt(Curl, CURLOPT_WRITEDATA, &Curl_Data2);
     curl_easy_setopt(Curl, CURLOPT_HTTPHEADER, HttpHeader);
     curl_easy_setopt(Curl, CURLOPT_URL, FileName_String.c_str());
+    if (Reader_Curl->SetOptions())
+        return 1;
     CURLcode Result=curl_easy_perform(Curl);
+    if (Result)
+    {
+        Reader_Curl->Curl_Log(Result);
+        return 1;
+    }
     curl_easy_setopt(Curl, CURLOPT_WRITEFUNCTION, NULL);
     curl_easy_setopt(Curl, CURLOPT_WRITEDATA, NULL);
     curl_easy_setopt(Curl, CURLOPT_HTTPHEADER, NULL);
@@ -388,12 +587,13 @@ string Amazon_AWS_GetRegion(const string &serviceName, const string &bucketName,
     curl_slist_free_all(HttpHeader);
 
     //We get the region name
-    return Curl_Data2.Amazon_AWS_Region;
+    RegionName=Curl_Data2.Amazon_AWS_Region;
+    return 0;
 }
 
-void Amazon_AWS_Manage(Http::Url &File_URL, Reader_libcurl::curl_data* Curl_Data, const ZtringList& ExternalHttpHeaders)
+size_t Amazon_AWS_Manage(Http::Url &File_URL, Reader_libcurl* Reader_Curl, const ZtringList& ExternalHttpHeaders)
 {
-    // Looking for style of URL (Virtual-hostedstyle URL Path-style URL), service name, region name
+    // Looking for style of URL (Virtual-hosted-style URL Path-style URL), service name, region name
     // Format:
     // <bucket>.<service>.<aws-region>
     // <service>.<aws-region>
@@ -416,7 +616,8 @@ void Amazon_AWS_Manage(Http::Url &File_URL, Reader_libcurl::curl_data* Curl_Data
         if (AfterLastDot)
         {
             string bucketName=regionName.substr(0, AfterLastDot-1); 
-            regionName=Amazon_AWS_GetRegion(serviceName, bucketName, File_URL, Curl_Data->Curl, ExternalHttpHeaders);
+            if (Amazon_AWS_GetRegion(serviceName, bucketName, File_URL, Reader_Curl, ExternalHttpHeaders, regionName))
+                return 1;
         }
         else
             regionName="us-east-1";
@@ -447,13 +648,15 @@ void Amazon_AWS_Manage(Http::Url &File_URL, Reader_libcurl::curl_data* Curl_Data
     // Handling S3 signature
     if (serviceName=="s3" && !regionName.empty())
     {
-        Amazon_AWS_Sign(Curl_Data->File_Name, Curl_Data->HttpHeader, File_URL, regionName, serviceName, ExternalHttpHeaders);
+        Amazon_AWS_Sign(Reader_Curl->Curl_Data->HttpHeader, File_URL, regionName, serviceName, ExternalHttpHeaders);
 
         //Removing the login/pass FTP-style, not usable by libcurl
         File_URL.User.clear();
         File_URL.Password.clear();
-        Curl_Data->File_Name=Ztring().From_UTF8(File_URL.ToString());
+        Reader_Curl->Curl_Data->File_Name=Ztring().From_UTF8(File_URL.ToString());
     }
+
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -473,7 +676,7 @@ size_t libcurl_WriteData_CallBack(void *ptr, size_t size, size_t nmemb, void *da
             long http_code = 0;
             if (curl_easy_getinfo (((Reader_libcurl::curl_data*)data)->Curl, CURLINFO_RESPONSE_CODE, &http_code)!=CURLE_OK || http_code != 200)
             {
-                MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(((Reader_libcurl::curl_data*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
+                MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPasswordAndParameters(((Reader_libcurl::curl_data*)data)->File_Name)+__T(", ")+Ztring().From_UTF8(string((char*)ptr, size*nmemb)));
                 return size*nmemb;
             }
         }
@@ -656,7 +859,7 @@ void Reader_libcurl::Curl_Log(int Result, const Ztring &Message)
 {
 #if MEDIAINFO_EVENTS
     if (Result == CURLE_UNKNOWN_TELNET_OPTION)
-        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0xF1010102, Reader_libcurl_FileNameWithoutPassword(Curl_Data->File_Name) + Message);
+        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0xF1010102, Reader_libcurl_FileNameWithoutPasswordAndParameters(Curl_Data->File_Name) + Message);
     else
     {
         Curl_Log(Result);
@@ -673,9 +876,119 @@ void Reader_libcurl::Curl_Log(int Result)
     MessageString.From_Local(Curl_Data->ErrorBuffer);
     if (MessageString.empty())
         MessageString.From_Local(curl_easy_strerror(CURLcode(Result)));
-    MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(Curl_Data->File_Name) + __T(", ") + MessageString);
+    int32u MessageCode;
+    switch (Result)
+    {
+    case CURLE_SSL_CACERT                   : MessageCode = 0xF1010105; break;
+    default                                 : MessageCode = 0;
+    }
+    MediaInfoLib::Config.Log_Send(0xC0, 0xFF, MessageCode, Reader_libcurl_FileNameWithoutPasswordAndParameters(Curl_Data->File_Name) + __T(", ") + MessageString);
 #endif
 }
+
+//---------------------------------------------------------------------------
+size_t Reader_libcurl::SetOptions()
+{
+    if (!Curl_Data->Ssl_CertificateFileName.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLCERT, Curl_Data->Ssl_CertificateFileName.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (!Curl_Data->Ssl_CertificateFormat.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLCERTTYPE, Curl_Data->Ssl_CertificateFormat.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (!Curl_Data->Ssl_PrivateKeyFileName.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLKEY, Curl_Data->Ssl_PrivateKeyFileName.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (!Curl_Data->Ssl_PrivateKeyFormat.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLKEYTYPE, Curl_Data->Ssl_PrivateKeyFormat.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (!Curl_Data->Ssl_CertificateAuthorityFileName.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_CAINFO, Curl_Data->Ssl_CertificateAuthorityFileName.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (!Curl_Data->Ssl_CertificateAuthorityPath.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_CAPATH, Curl_Data->Ssl_CertificateAuthorityPath.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+    /*else
+    {
+        CURLcode res;
+        char* cainfo = NULL;
+        #define CURLINFO_STRING   0x100000
+        CURLINFO CURLINFO_CAINFO = (CURLINFO)(CURLINFO_STRING + 61);
+        curl_easy_getinfo(Curl_Data->Curl, CURLINFO_CAINFO, &cainfo);
+        if (cainfo)
+            fprintf(stderr, "default ca info path: %s\n", cainfo);
+    }*/
+
+    if (!Curl_Data->Ssl_CertificateRevocationListFileName.empty())
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_CRLFILE, Curl_Data->Ssl_CertificateRevocationListFileName.c_str());
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    if (Curl_Data->Ssl_IgnoreSecurity)
+    {
+        CURLcode Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSL_VERIFYPEER, 0);
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+
+        Result = curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSL_VERIFYHOST, 0);
+        if (Result)
+        {
+            Curl_Log(Result);
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 size_t Reader_libcurl::Format_Test_PerParser(MediaInfo_Internal* MI, const String &File_Name)
 {
@@ -764,25 +1077,55 @@ size_t Reader_libcurl::Format_Test_PerParser(MediaInfo_Internal* MI, const Strin
                 Curl_Data->HttpHeader=curl_slist_append(Curl_Data->HttpHeader, HttpHeaderStrings[Pos].To_Local().c_str());
     }
 
+    // URL encoding
+    Http::Url File_URL=Http::Url(Ztring(Curl_Data->File_Name).To_UTF8());
+    MediaInfo_Config::urlencode UrlEncode=Config.URLEncode_Get();
+    if (UrlEncode==MediaInfo_Config::URLEncode_Guess)
+    {
+        UrlEncode=DetectPercentEncode(File_URL.Path, true);
+        if (UrlEncode!=MediaInfo_Config::URLEncode_No && DetectPercentEncode(File_URL.Host)==MediaInfo_Config::URLEncode_No)
+            UrlEncode=MediaInfo_Config::URLEncode_No;
+        if (UrlEncode!=MediaInfo_Config::URLEncode_No)
+        {
+            ZtringListList List;
+            List.Separator_Set(0, "&");
+            List.Separator_Set(1, "=");
+            for (size_t i=0; i<List.size(); i++)
+                for (size_t j=0; j<List[i].size(); j++)
+                    if (DetectPercentEncode(File_URL.Query)==MediaInfo_Config::URLEncode_No)
+                        UrlEncode=MediaInfo_Config::URLEncode_No;
+        }
+        if (UrlEncode!=MediaInfo_Config::URLEncode_No && DetectPercentEncode(File_URL.Fragment)==MediaInfo_Config::URLEncode_No)
+            UrlEncode=MediaInfo_Config::URLEncode_No;
+    }
+    if (UrlEncode==MediaInfo_Config::URLEncode_No)
+    {
+        File_URL.User=PercentEncode(File_URL.User);
+        File_URL.Password=PercentEncode(File_URL.Password);
+        File_URL.Host=PercentEncode(File_URL.Host);
+        File_URL.Path=PercentEncode(File_URL.Path);
+        File_URL.Query=PercentEncode(File_URL.Query);
+        File_URL.Fragment=PercentEncode(File_URL.Fragment);
+        Curl_Data->File_Name=Ztring().From_UTF8(File_URL.ToString());
+        UrlEncode=MediaInfo_Config::URLEncode_Yes;
+    }
+
     //Special cases
-    Http::Url File_URL=Http::Url(Ztring(File_Name).To_UTF8());
     if (!File_URL.Protocol.empty())
     {
-        // URL encoding
-        MediaInfo_Config::urlencode ShouldUrlEncode=Config.URLEncode_Get();
-        if (ShouldUrlEncode==MediaInfo_Config::URLEncode_Guess)
-        {
-            if (File_URL.Path.find(' '))
-                ShouldUrlEncode=MediaInfo_Config::URLEncode_Yes;
-        }
-        if (ShouldUrlEncode==MediaInfo_Config::URLEncode_Yes)
-            File_URL.Path=ZenLib::Format::Http::URL_Encoded_Encode(File_URL.Path);
-
         // Amazon S3 specific credentials
         if ((File_URL.Protocol=="http" || File_URL.Protocol=="https") && !File_URL.User.empty())
         {
             if (File_URL.Host.find(".amazonaws.com")+14==File_URL.Host.size())
-                Amazon_AWS_Manage(File_URL, Curl_Data, HttpHeaderStrings);
+            {
+                if (UrlEncode==MediaInfo_Config::URLEncode_Yes)
+                {
+                    File_URL.User=PercentDecode(File_URL.User);
+                    File_URL.Password=PercentDecode(File_URL.Password);
+                }
+                if (Amazon_AWS_Manage(File_URL, this, HttpHeaderStrings))
+                    return 0;
+            }
         }
 
         if (File_URL.Protocol=="sftp" || File_URL.Protocol=="scp")
@@ -831,95 +1174,12 @@ size_t Reader_libcurl::Format_Test_PerParser(MediaInfo_Internal* MI, const Strin
 
         if (File_URL.Protocol=="https" || File_URL.Protocol=="ftps")
         {
-            if (!Curl_Data->Ssl_CertificateFileName.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLCERT, Curl_Data->Ssl_CertificateFileName.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_CertificateFormat.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLCERTTYPE, Curl_Data->Ssl_CertificateFormat.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_PrivateKeyFileName.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLKEY, Curl_Data->Ssl_PrivateKeyFileName.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_PrivateKeyFormat.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSLKEYTYPE, Curl_Data->Ssl_PrivateKeyFormat.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_CertificateAuthorityFileName.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_CAINFO, Curl_Data->Ssl_CertificateAuthorityFileName.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_CertificateAuthorityPath.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_CAPATH, Curl_Data->Ssl_CertificateAuthorityPath.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (!Curl_Data->Ssl_CertificateRevocationListFileName.empty())
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_CRLFILE, Curl_Data->Ssl_CertificateRevocationListFileName.c_str());
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
-
-            if (Curl_Data->Ssl_IgnoreSecurity)
-            {
-                CURLcode Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSL_VERIFYPEER, 0);
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-
-                Result=curl_easy_setopt(Curl_Data->Curl, CURLOPT_SSL_VERIFYHOST, 0);
-                if (Result)
-                {
-                    Curl_Log(Result);
-                    return 0;
-                }
-            }
+            if (SetOptions())
+                return 0;
         }
     }
     string FileName_String=Ztring(Curl_Data->File_Name).To_UTF8();
+    Curl_Data->File_Name=File_Name;
     curl_easy_setopt(Curl_Data->Curl, CURLOPT_URL, FileName_String.c_str());
     curl_easy_setopt(Curl_Data->Curl, CURLOPT_FOLLOWLOCATION, 1);
     curl_easy_setopt(Curl_Data->Curl, CURLOPT_MAXREDIRS, 3);
@@ -1217,7 +1477,7 @@ size_t Reader_libcurl::Format_Test_PerParser_Continue (MediaInfo_Internal* MI)
                         }
                     }
                     Curl_Data->ErrorBuffer[0]='\0';
-                    MediaInfoLib::Config.Log_Send(0xC0, 0xFF, MessageCode, Reader_libcurl_FileNameWithoutPassword(Curl_Data->File_Name)+__T(", ")+MessageString);
+                    MediaInfoLib::Config.Log_Send(0xC0, 0xFF, MessageCode, Reader_libcurl_FileNameWithoutPasswordAndParameters(Curl_Data->File_Name)+__T(", ")+MessageString);
                 #endif //MEDIAINFO_EVENTS
             }
 
@@ -1280,13 +1540,23 @@ bool Reader_libcurl::Load(const Ztring File_Name)
             size_t Errors=0;
 
             /* Load library */
+            #if defined(__APPLE__) && defined(__MACH__)
+                #define MEDIAINFODLL_NAME  "libcurl.4.dylib"
+                #define __stdcall
+            #elif !defined (_WIN32) && !defined (WIN32)
+                #define MEDIAINFODLL_NAME  "libcurl.so.4"
+                #define __stdcall
+            #endif //!defined(_WIN32) || defined (WIN32)
             #ifdef MEDIAINFO_GLIBC
                 libcurl_Module=g_module_open(MEDIAINFODLL_NAME, G_MODULE_BIND_LAZY);
             #elif defined (_WIN32) || defined (WIN32)
                 #ifdef WINDOWS_UWP
-                    libcurl_Module=LoadPackagedLibrary(MEDIAINFODLL_NAME, 0);
+                    libcurl_Module=LoadPackagedLibrary(__T("libcurl.dll"), 0);
                 #else
-                    libcurl_Module=LoadLibrary(MEDIAINFODLL_NAME);
+                    if (sizeof(size_t)==8)
+                        libcurl_Module=LoadLibrary(__T("libcurl-x64.dll"));
+                    if (!libcurl_Module)
+                        libcurl_Module=LoadLibrary(__T("libcurl.dll"));
                 #endif
             #else
                 libcurl_Module=dlopen(MEDIAINFODLL_NAME, RTLD_LAZY);
@@ -1305,7 +1575,7 @@ bool Reader_libcurl::Load(const Ztring File_Name)
             {
                 #if MEDIAINFO_EVENTS
                     if (!File_Name.empty())
-                        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(File_Name)+__T(", Libcurl library is not found"));
+                        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPasswordAndParameters(File_Name)+__T(", Libcurl library is not found"));
                 #endif //MEDIAINFO_EVENTS
                 return 0;
             }
@@ -1330,7 +1600,7 @@ bool Reader_libcurl::Load(const Ztring File_Name)
             {
                 #if MEDIAINFO_EVENTS
                     if (!File_Name.empty())
-                        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPassword(File_Name)+__T(", Libcurl library is not correctly loaded"));
+                        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, Reader_libcurl_FileNameWithoutPasswordAndParameters(File_Name)+__T(", Libcurl library is not correctly loaded"));
                 #endif //MEDIAINFO_EVENTS
                 return 0;
             }
